/**
 * Copyright (c) 2021, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { TestableComponentInterface } from "@wso2is/core/models";
import { Field, Form } from "@wso2is/form";
import isEmpty from "lodash-es/isEmpty";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import {
    ConnectorPropertyInterface,
    GovernanceConnectorInterface,
    ServerConfigurationsConstants
} from "../../../../features/server-configurations";

/**
 * Interface for Self Registration Configuration Form props.
 */
interface SelfRegistrationConfigurationFormPropsInterface extends TestableComponentInterface {
    /**
     * Connector's initial values.
     */
    initialValues: GovernanceConnectorInterface;
    /**
     * Callback for form submit.
     * @param {any} values - Resolved Form Values.
     */
    onSubmit: (values) => void;
    /**
     * Is readonly.
     */
    readOnly?: boolean;
}

const NOTIFY_ACCOUNT_CONFIRMATION: string = "SelfRegistration.NotifyAccountConfirmation";

const allowedConnectorFields: string[] = [
    ServerConfigurationsConstants.SELF_REGISTRATION_ENABLE,
    ServerConfigurationsConstants.SELF_SIGN_UP_NOTIFICATIONS_INTERNALLY_MANAGED,
    ServerConfigurationsConstants.VERIFICATION_CODE_EXPIRY_TIME,
    NOTIFY_ACCOUNT_CONFIRMATION
];

/**
 * Self Registration Configuration Form.
 *
 * @param {SelfRegistrationConfigurationFormPropsInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
export const SelfRegistrationConfigurationForm: FunctionComponent<SelfRegistrationConfigurationFormPropsInterface> = (
    props: SelfRegistrationConfigurationFormPropsInterface
): ReactElement => {

    const {
        initialValues,
        onSubmit,
        readOnly,
        ["data-testid"]: testId
    } = props;

    const { t } = useTranslation();

    const [ initialConnectorValues, setInitialConnectorValues ]
        = useState<Map<string, ConnectorPropertyInterface>>(undefined);
    const [ showExpiryTime, setShowExpiryTime ] = useState<boolean>(false);

    /**
     * Flattens and resolved form initial values and field metadata.
     */
    useEffect(() => {

        if (isEmpty(initialValues?.properties)) {
            return;
        }

        const resolvedInitialValues: Map<string, ConnectorPropertyInterface>
            = new Map<string, ConnectorPropertyInterface>();

        initialValues.properties.map((property: ConnectorPropertyInterface) => {
            if (allowedConnectorFields.includes(property.name)) {
                resolvedInitialValues.set(property.name, property);
            }
            if (property.name === NOTIFY_ACCOUNT_CONFIRMATION && property.value === "true") {
                setShowExpiryTime(true);
            }
        });
        setInitialConnectorValues(resolvedInitialValues);
    }, [ initialValues ]);

    /**
     * Prepare form values for submitting.
     *
     * @param values - Form values.
     *
     * @return Sanitized form values.
     */
    const getUpdatedConfigurations = (values) => {
        let data: any = {
            "SelfRegistration.Enable": values.enable !== undefined
                ? !!values.enable
                : initialConnectorValues?.get("SelfRegistration.Enable").value,
            "SelfRegistration.NotifyAccountConfirmation": values.signUpConfirmation !== undefined
                ? !!values.signUpConfirmation
                : initialConnectorValues?.get("SelfRegistration.NotifyAccountConfirmation").value,
            "SelfRegistration.VerificationCode.ExpiryTime": values.verificationLinkExpiryTime !== undefined
                ? values.verificationLinkExpiryTime
                : initialConnectorValues?.get("SelfRegistration.VerificationCode.ExpiryTime").value,
            "SelfRegistration.LockOnCreation": values.signUpConfirmation !== undefined
                ? !!values.signUpConfirmation
                : initialConnectorValues?.get("SelfRegistration.NotifyAccountConfirmation").value
        };

        if (initialConnectorValues?.get("SelfRegistration.Notification.InternallyManage").value === "false" &&
            values.signUpConfirmation !== undefined && !!values.signUpConfirmation) {
            data = {
                ...data,
                "SelfRegistration.Notification.InternallyManage": true
            };
        }

        return data;
    };

    return (
        initialConnectorValues ?
            <Form
                uncontrolledForm={false}
                onSubmit={ (values) => onSubmit(getUpdatedConfigurations(values)) }
            >
                <Field.Checkbox
                    ariaLabel="selfRegistrationEnable"
                    name="enable"
                    label={ t("extensions:manage.serverConfigurations.userOnboarding." +
                        "selfRegistration.form.fields.enable.label") }
                    required={ false }
                    value={ initialConnectorValues?.get(
                        ServerConfigurationsConstants.SELF_REGISTRATION_ENABLE)?.value === "true" ? ["enable"] : [] }
                    readOnly={ readOnly }
                    data-testid={ `${testId}-self-registration-enable` }
                    hint={ t("extensions:manage.serverConfigurations.userOnboarding." +
                        "selfRegistration.form.fields.enable.hint") }
                />
                <Field.Checkbox
                    ariaLabel="signUpConfirmation"
                    name="signUpConfirmation"
                    label={ t("extensions:manage.serverConfigurations.userOnboarding." +
                        "selfRegistration.form.fields.signUpConfirmation.label") }
                    listen={ (value) => setShowExpiryTime(value) }
                    required={ false }
                    value={ initialConnectorValues?.get(
                        NOTIFY_ACCOUNT_CONFIRMATION)?.value === "true" ? ["signUpConfirmation"] : [] }
                    readOnly={ readOnly }
                    data-testid={ `${testId}-selfRegistration-notify-accountConfirmation` }
                    hint={ t("extensions:manage.serverConfigurations.userOnboarding." +
                        "selfRegistration.form.fields.signUpConfirmation.hint") }

                />
                { showExpiryTime &&
                <Field.Input
                    ariaLabel="verificationLinkExpiryTime"
                    inputType="default"
                    name="verificationLinkExpiryTime"
                    type="number"
                    min="0"
                    label={ t("extensions:manage.serverConfigurations.userOnboarding." +
                        "selfRegistration.form.fields.expiryTime.label") }
                    required={ false }
                    placeholder={ t("console:manage.features.claims.local.forms." +
                        "displayOrder.placeholder") }
                    value={ initialConnectorValues?.get(
                        ServerConfigurationsConstants.VERIFICATION_CODE_EXPIRY_TIME)?.value.toString() || 0 }
                    maxLength={ 50 }
                    minLength={ 1 }
                    data-testid={ `${testId}-form-display-order-input` }
                    hint={ initialConnectorValues?.get(
                        ServerConfigurationsConstants.VERIFICATION_CODE_EXPIRY_TIME)?.description }
                />
                }
                <Field.Button
                    size="small"
                    buttonType="primary_btn"
                    ariaLabel="Self registration update button"
                    name="update-button"
                    data-testid={ `${testId}-submit-button` }
                    disabled={ false }
                    label={ t("common:update") }
                    hidden={ readOnly }
                />
            </Form>
            : null
    );
};

/**
 * Default props for the component.
 */
SelfRegistrationConfigurationForm.defaultProps = {
    "data-testid": "self-registration-edit-form"
};
